#!/usr/bin/env node

let co = require('co')
let commander = require('commander')
let request = require('request')
let fs = require('fs')
let prompt = require('co-prompt')
let os = require('os')

// load config from file
let configFile = os.homedir() + '/.count2k/config.json'
let configData = fs.readFileSync(configFile)
if (!configData) { 
  return console.log('error loading configuration')
}
configJSON = JSON.parse(configData)

// this is the endpoint we are connecting to
let endpoint = configJSON.endpoint
let username = configJSON.username
let password = configJSON.password

commander.version('0.1.0')

commander.command('update')
  .description('update the function for an existing counter')
  .action(function (id, filename) { 

    // base64 encode the actual file 
    fs.readFile(process.cwd() + '/' + filename, 'utf8', function (err, data) { 

      var data = {
        function: new Buffer(data).toString('base64')
      }

      request({ 
        url: endpoint + '/counters/' + id,
        auth: {
          user: username,
          pass: password
        },
        method: 'POST',
        json: data
      }, function (err, resp, body) { 

        if (!err && resp.statusCode == 200) {
          console.log('counter updated successfully. Current Reading: ', body.value, '/',body.max)
        } else { 
          console.log('counter update failed. Please try again')
        }
        process.exit(0)

      })

    })

  })

// list existing counters by id 
commander.command('list')
  .description('list all the counters')
  .action(function () { 

    request({ 
      url: endpoint + '/counters',
      method: 'GET',
      auth: {
        user: username,
        pass: password
      },      
      json: true
    }, function (err, resp, body) { 

      if (err) { 

        return console.log(err)

      }

      if (!body) { 
        return console.log(body)
      }

      body.forEach(function (counter) { 

        console.log(counter.id, counter.short, counter.value, counter.max)

      })

    })

  })

commander.command('remove [id]')
  .description('remove a counter by id')
  .action(function (id) { 

    request({ 
      url: endpoint + '/counters/' + id,
      auth: {
        user: username,
        pass: password
      },
      method: 'DELETE'
    }, function (err, resp, body) { 

      if (!err && resp.statusCode == 200) { 
        console.log('counter removed successfully')
      } else { 
        console.log('error removing counter')
      }

    })
    
  })

// test a counter 
commander.command('test [file.js]')
  .description('test a javascript function')
  .action(function (filename) { 

    // base64 encode the actual file 
    fs.readFile(process.cwd() + '/' + filename, 'utf8', function (err, data) { 

      var data = {
        function: new Buffer(data).toString('base64')
      }

      request({ 
        url: endpoint + '/test',
        method: 'POST',
        auth: {
          user: username,
          pass: password
        },
        timeout: 120000,
        json: data
      }, function (err, resp, body) { 

        if (!err && resp.statusCode == 200) {
          console.log('Counter value: ', body.value)
        } else { 
          console.log('Error deploying counter')
        }
        process.exit(0)

      })

    })

  })

// create a new counter
commander.command('create [file.js]')
  .description('create a new counter from a javscript function')
  .action(function (file) { 
    co(function *() {
    
      // prompt for a sensor short name
      var short = yield prompt('title: ')

      // prompt for the sensors maximum value
      var max = yield prompt('count to: ')

      var r = yield prompt('red: ')
      var g = yield prompt('green: ')
      var b = yield prompt('blue: ')

      // base64 encode the actual file 
      fs.readFile(process.cwd() + '/' + file, 'utf8', function (err, data) { 

        var data = {
          short: short,
          long: short,
          function: new Buffer(data).toString('base64'),
          max: max,
          push: true,
          color: { 
            r: r,
            g: g,
            b: b
          }
        }

        // do a post request with the body as a json object
        request({ 
          url: endpoint + '/counters',
          method: 'POST',
          auth: {
            user: username,
            pass: password
          },
          json: data
        }, function (err, resp, body) { 

          // console.log(err)
          // console.log(resp.statusCode)
          if (!err && resp.statusCode == 200) {
            console.log('counter created successfully. Current Reading: ', body.value, '/',body.max)
          } else { 
            console.log('counter creation failed. Please try again')
          }
          process.exit(0)

        })

        // return success or failure

      })    

    })

  })

commander.parse(process.argv);